<?php

use CRM_FormProcessor_ExtensionUtil as E;

/**
 * @author Jaap Jansma (CiviCooP) <jaap.jansma@civicoop.org>
 * @license http://www.gnu.org/licenses/agpl-3.0.html
 */
class CRM_FormProcessor_BAO_FormProcessorDefaultDataAction extends CRM_FormProcessor_DAO_FormProcessorDefaultDataAction {
  
	 /**
   * Function to get values
   * 
   * @return array $result found rows with data
   * @access public
   * @static
   */
  public static function getValues($params) {
    $result = array();
    $action = new CRM_FormProcessor_BAO_FormProcessorDefaultDataAction();
    if (!empty($params)) {
      $fields = self::fields();
      foreach ($params as $key => $value) {
        if (isset($fields[$key])) {
          $action->$key = $value;
        }
      }
    }
		$action->orderBy('weight');
    $action->find();
    while ($action->fetch()) {
      $row = array();
      self::storeValues($action, $row);
      if (!empty($row['form_processor_instance_id'])) {
      	$provider = form_processor_get_action_provider();
				$action_type = $provider->getActionByName($row['type']);
				
				if ($action_type) {
					$configuration = $action_type->getDefaultConfiguration();
					if (isset($row['configuration']) && is_string($row['configuration'])) {
						$row['configuration'] = json_decode($row['configuration'], true);
					}
					if (empty($row['configuration']) || !is_array($row['configuration'])) {
						$row['configuration'] = array();
					}
					foreach($row['configuration'] as $name => $value) {
						$configuration->setParameter($name, $value);
					}
					$action_type->setConfiguration($configuration);
					
					$row['type'] = $action_type;
				}
				
				if (isset($row['mapping']) && is_string($row['mapping'])) {
						$row['mapping'] = json_decode($row['mapping'], true);
					}
					if (empty($row['mapping']) || !is_array($row['mapping'])) {
						$row['mapping'] = array();
					}
				
        $result[$row['id']] = $row;
      } else {
        //invalid input because no there is no form processor
        CRM_FormProcessor_BAO_FormProcessorDefaultDataAction::deleteWithId($row['id']);
      }
    }
    return $result;
  }

  /**
   * Function to add or update form processor action
   * 
   * @param array $params 
   * @return array $result
   * @access public
   * @throws Exception when params is empty
   * @static
   */
  public static function add($params) {
    $result = array();
    if (empty($params)) {
      throw new Exception('Params can not be empty when adding or updating a form processor action');
    }
		
		if (!empty($params['id'])) {
      CRM_Utils_Hook::pre('edit', 'FormProcessorDefaultDataAction', $params['id'], $params);
    }
    else {
      CRM_Utils_Hook::pre('create', 'FormProcessorDefaultDataAction', NULL, $params);
    }
		
    $action = new CRM_FormProcessor_BAO_FormProcessorDefaultDataAction();
    $fields = self::fields();
    foreach ($params as $key => $value) {
      if (isset($fields[$key])) {
        $action->$key = $value;
      }
    }

		if (is_array($action->configuration)) {
			$action->configuration = json_encode($action->configuration);
		}
		if (is_array($action->mapping)) {
			$action->mapping = json_encode($action->mapping);
		}
    $action->save();
    self::storeValues($action, $result);
		
		if (!empty($params['id'])) {
      CRM_Utils_Hook::post('edit', 'FormProcessorDefaultDataAction', $action->id, $action);
    }
    else {
      CRM_Utils_Hook::post('create', 'FormProcessorDefaultDataAction', $action->id, $action);
    }
		
    return $result;
  }

  /**
   * Function to delete a form processor action with id
   * 
   * @param int $id
   * @throws Exception when $id is empty
   * @access public
   * @static
   */
  public static function deleteWithId($id) {
    if (empty($id)) {
      throw new Exception('id can not be empty when attempting to delete a form processor action');
    }
		
		CRM_Utils_Hook::pre('delete', 'FormProcessorDefaultDataAction', $id, CRM_Core_DAO::$_nullArray);
		
    $action = new CRM_FormProcessor_BAO_FormProcessorDefaultDataAction();
    $action->id = $id;
    if ($action->find(true)) {
      $action->delete();
    }
		
		CRM_Utils_Hook::post('delete', 'FormProcessorDefaultDataAction', $id, CRM_Core_DAO::$_nullArray);
		
    return;
  }

	/**
   * Function to delete all actions with a form processor instance id
   *
   * @param int $formProcessorInstanceId
   * @access public
   * @static
   */
  public static function deleteWithFormProcessorInstanceId($formProcessorInstanceId) {
    $action = new CRM_FormProcessor_BAO_FormProcessorDefaultDataAction();
    $action->form_processor_instance_id = $formProcessorInstanceId;
    $action->find(FALSE);
    while ($action->fetch()) {
      CRM_Utils_Hook::pre('delete', 'FormProcessorDefaultDataAction', $action->id, CRM_Core_DAO::$_nullArray);
		
      $action->delete();
			
			CRM_Utils_Hook::post('delete', 'FormProcessorDefaultDataAction', $action->id, CRM_Core_DAO::$_nullArray);
    }
  }

	/**
   * Public function to generate name from title
   *
   * @param $title
   * @return string
   * @access public
   * @static
   */
  public static function buildNameFromTitle($title) {
    $titleParts = explode(' ', strtolower($title));
    $nameString = implode('_', $titleParts);
    return substr($nameString, 0, 80);
  }
	
	/**
	 * Returns whether the name is valid or not
	 * 
	 * @param string $name
	 * @param int $id optional
	 * @return bool
	 * @static
	 */
	public static function isNameValid($name, $id=null) {
		$sql = "SELECT COUNT(*) FROM `civicrm_form_processor_default_data_action` WHERE `name` = %1";
		$params[1] = array($name, 'String');
		if ($id) {
			$sql .= " AND `id` != %2";
			$params[2] = array($id, 'Integer');
		}
		$count = CRM_Core_DAO::singleValueQuery($sql, $params);
		return ($count > 0) ? false : true;
	}
	
	
}