<?php

use CRM_FormProcessor_ExtensionUtil as E;

/**
 * @author Jaap Jansma (CiviCooP) <jaap.jansma@civicoop.org>
 * @license http://www.gnu.org/licenses/agpl-3.0.html
 */
class CRM_FormProcessor_BAO_FormProcessorInputValidation extends CRM_FormProcessor_DAO_FormProcessorInputValidation {
  
	 /**
   * Function to get values
   * 
   * @return array $result found rows with data
   * @access public
   * @static
   */
  public static function getValues($params) {
    $result = array();
    $inputValidation = new CRM_FormProcessor_BAO_FormProcessorInputValidation();
    if (!empty($params)) {
      $fields = self::fields();
      foreach ($params as $key => $value) {
        if (isset($fields[$key])) {
          $inputValidation->$key = $value;
        }
      }
    }
    $inputValidation->find();
    while ($inputValidation->fetch()) {
      $row = array();
      self::storeValues($inputValidation, $row);
      if (!empty($row['form_processor_input_id'])) {
      	$validator = \Civi::service('form_processor_validation_factory')->getValidatorByName($row['validator']);
				if ($validator) {
					$configuration = $validator->getDefaultConfiguration();
					if (isset($row['configuration']) && is_string($row['configuration'])) {
						$row['configuration'] = json_decode($row['configuration'], true);
					}
					if (empty($row['configuration']) || !is_array($row['configuration'])) {
						$row['configuration'] = array();
					}
					foreach($row['configuration'] as $name => $value) {
						$configuration->set($name, $value);
					}
					$validator->setConfiguration($configuration);
					
					$row['validator'] = $validator;
				}
        $result[$row['id']] = $row;
      } else {
        //invalid input because no there is no form processor
        CRM_FormProcessor_BAO_FormProcessorInputValidation::deleteWithId($row['id']);
      }
    }
    return $result;
  }

  /**
   * Function to add or update form processor input validation
   * 
   * @param array $params 
   * @return array $result
   * @access public
   * @throws Exception when params is empty
   * @static
   */
  public static function add($params) {
    $result = array();
    if (empty($params)) {
      throw new Exception('Params can not be empty when adding or updating a form processor input validation');
    }
		
		if (!empty($params['id'])) {
      CRM_Utils_Hook::pre('edit', 'FormProcessorInputValidation', $params['id'], $params);
    }
    else {
      CRM_Utils_Hook::pre('create', 'FormProcessorInputValidation', NULL, $params);
    }
		
    $inputValidation = new CRM_FormProcessor_BAO_FormProcessorInputValidation();
    $fields = self::fields();
    foreach ($params as $key => $value) {
      if (isset($fields[$key])) {
        $inputValidation->$key = $value;
      }
    }
		if (is_array($inputValidation->configuration)) {
			$inputValidation->configuration = json_encode($inputValidation->configuration);
		}
    $inputValidation->save();
    self::storeValues($inputValidation, $result);
		
		if (!empty($params['id'])) {
      CRM_Utils_Hook::post('edit', 'FormProcessorInputValidation', $inputValidation->id, $inputValidation);
    }
    else {
      CRM_Utils_Hook::post('create', 'FormProcessorInputValidation', $inputValidation->id, $inputValidation);
    }
		
    return $result;
  }

  /**
   * Function to delete a form processor input with id
   * 
   * @param int $id
   * @throws Exception when $id is empty
   * @access public
   * @static
   */
  public static function deleteWithId($id) {
    if (empty($id)) {
      throw new Exception('id can not be empty when attempting to delete a form processor input validation');
    }
		
		CRM_Utils_Hook::pre('delete', 'FormProcessorInputValidation', $id, CRM_Core_DAO::$_nullArray);
		
    $inputValidation = new CRM_FormProcessor_BAO_FormProcessorInputValidation();
    $inputValidation->id = $id;
    if ($inputValidation->find(true)) {
      $inputValidation->delete();
    }
		
		CRM_Utils_Hook::post('delete', 'FormProcessorInputValidation', $id, CRM_Core_DAO::$_nullArray);
		
    return;
  }

	/**
   * Function to delete all inputs with a form processor input id
   *
   * @param int $formProcessorInputId
   * @access public
   * @static
   */
  public static function deleteWithFormProcessorInputId($formProcessorInputId) {
    $inputValidation = new CRM_FormProcessor_BAO_FormProcessorInputValidation();
    $inputValidation->form_processor_input_id = $formProcessorInputId;
    $inputValidation->find(FALSE);
    while ($inputValidation->fetch()) {
      CRM_Utils_Hook::pre('delete', 'FormProcessorInputValidation', $inputValidation->id, CRM_Core_DAO::$_nullArray);
		
      $inputValidation->delete();
			
			CRM_Utils_Hook::post('delete', 'FormProcessorInputValidation', $inputValidation->id, CRM_Core_DAO::$_nullArray);
    }
  }
	
	
}