<?php
/**
 * @author Jaap Jansma (CiviCooP) <jaap.jansma@civicoop.org>
 * @license http://www.gnu.org/licenses/agpl-3.0.html
 */
 
 class CRM_FormProcessor_BAO_FormProcessorInstance extends CRM_FormProcessor_DAO_FormProcessorInstance {
 	
	/**
   * Function to get values
   * 
   * @return array $result found rows with data
   * @access public
   * @static
   */
  public static function getValues($params) {
    $result = array();
    $formProcessorInstance = new CRM_FormProcessor_BAO_FormProcessorInstance();
    if (!empty($params)) {
      $fields = self::fields();
      foreach ($params as $key => $value) {
        if (isset($fields[$key])) {
          $formProcessorInstance->$key = $value;
        }
      }
    }
    $formProcessorInstance->find();
    while ($formProcessorInstance->fetch()) {
      $row = array();
      self::storeValues($formProcessorInstance, $row); 
			$row['inputs'] = array_values(CRM_FormProcessor_BAO_FormProcessorInput::getValues(array('form_processor_instance_id' => $formProcessorInstance->id)));
			$row['actions'] = array_values(CRM_FormProcessor_BAO_FormProcessorAction::getValues(array('form_processor_instance_id' => $formProcessorInstance->id)));
			$row['default_data_actions'] = array_values(CRM_FormProcessor_BAO_FormProcessorDefaultDataAction::getValues(array('form_processor_instance_id' => $formProcessorInstance->id)));
			
			$handler = \Civi::service('form_processor_output_handler_factory')->getHandlerByName($row['output_handler']);
			if ($handler) {
				$configuration = $handler->getDefaultConfiguration();
				if (isset($row['output_handler_configuration']) && is_string($row['output_handler_configuration'])) {
					$row['output_handler_configuration'] = json_decode($row['output_handler_configuration'], true);
				}
				if (empty($row['output_handler_configuration']) || !is_array($row['output_handler_configuration'])) {
					$row['output_handler_configuration'] = array();
				}
				foreach($row['output_handler_configuration'] as $name => $value) {
					$configuration->set($name, $value);
				}
				$handler->setConfiguration($configuration);
				
				$row['output_handler'] = $handler;
			}
			
			
      $result[$row['id']] = $row;
    }
    return $result;
  }
 	
	/**
   * Function to add or update a FormProcessorInstance
   * 
   * @param array $params 
   * @return array $result
   * @access public
   * @throws Exception when params is empty
   * @static
   */
  public static function add($params) {
    $result = array();
    if (empty($params)) {
      throw new Exception('Params can not be empty when adding or updating a civirule rule');
    }

    if (!empty($params['id'])) {
      CRM_Utils_Hook::pre('edit', 'FormProcessorInstance', $params['id'], $params);
    }
    else {
      CRM_Utils_Hook::pre('create', 'FormProcessorInstance', NULL, $params);
    }

    $formProcessorInstance = new CRM_FormProcessor_BAO_FormProcessorInstance();
    $fields = self::fields();
    foreach ($params as $key => $value) {
      if (isset($fields[$key])) {
        $formProcessorInstance->$key = $value;
      }
    }
    if (!isset($formProcessorInstance->name) || empty($formProcessorInstance->name)) {
      if (isset($formProcessorInstance->title)) {
        $formProcessorInstance->name = CRM_FormProcessor_BAO_FormProcessorInstance::buildNameFromLabel($formProcessorInstance->title);
      }
    }
		if (is_array($formProcessorInstance->output_handler_configuration)) {
			$formProcessorInstance->output_handler_configuration = json_encode($formProcessorInstance->output_handler_configuration);
		}
    $formProcessorInstance->save();
    self::storeValues($formProcessorInstance, $result);

    if (!empty($params['id'])) {
      CRM_Utils_Hook::post('edit', 'FormProcessorInstance', $formProcessorInstance->id, $formProcessorInstance);
    }
    else {
      CRM_Utils_Hook::post('create', 'FormProcessorInstance', $formProcessorInstance->id, $formProcessorInstance);
    }

    return $result;
  }

	/**
   * Function to delete a FormProcessorInstance with id
   * 
   * @param int $id
   * @throws Exception when $id is empty
   * @access public
   * @static
   */
  public static function deleteWithId($id) {
    if (empty($id)) {
      throw new Exception('id can not be empty when attempting to delete a form processor');
    }

    CRM_Utils_Hook::pre('delete', 'FormProcessorInstance', $id, CRM_Core_DAO::$_nullArray);

		// First delete all inputs
		CRM_FormProcessor_BAO_FormProcessorInput::deleteWithFormProcessorInstanceId($id);
		// And all actions
		CRM_FormProcessor_BAO_FormProcessorAction::deleteWithFormProcessorInstanceId($id);
		CRM_FormProcessor_BAO_FormProcessorDefaultDataAction::deleteWithFormProcessorInstanceId($id);

    $formProcessorInstance = new CRM_FormProcessor_BAO_FormProcessorInstance();
    $formProcessorInstance->id = $id;
    $formProcessorInstance->delete();

    CRM_Utils_Hook::post('delete', 'FormProcessorInstance', $id, CRM_Core_DAO::$_nullArray);

    return;
  }

	/**
   * Public function to generate name from label
   *
   * @param $label
   * @return string
   * @access public
   * @static
   */
  public static function buildNameFromLabel($label) {
    $labelParts = explode(' ', strtolower($label));
    $nameString = implode('_', $labelParts);
    return substr($nameString, 0, 80);
  }
	
	/**
	 * Returns whether the name is valid or not
	 * 
	 * @param string $name
	 * @param int $id optional
	 * @return bool
	 * @static
	 */
	public static function isNameValid($name, $id=null) {
		$invalidNames = array('getactions', 'getfields');
		if (in_array(strtolower($name), $invalidNames)) {
			return false;
		}
		$sql = "SELECT COUNT(*) FROM `civicrm_form_processor_instance` WHERE `name` = %1";
		$params[1] = array($name, 'String');
		if ($id) {
			$sql .= " AND `id` != %2";
			$params[2] = array($id, 'Integer');
		}
		$count = CRM_Core_DAO::singleValueQuery($sql, $params);
		return ($count > 0) ? false : true;
	}
	
 }
